local M = {}

-- Creates a new camera instance.
function M.newCamera()
    local c = {}
    c.x = 0
    c.y = 0
    c.w = love.graphics.getWidth()
    c.h = love.graphics.getHeight()
    c.centerX = c.x + c.w / 2
    c.centerY = c.y + c.h / 2
    return c
end

-- Enables the camera, which applies all the camera transformations to future draw calls.
function M.enable(cam)
    love.graphics.push()
    love.graphics.translate(cam.centerX, cam.centerY)
    love.graphics.translate(-cam.x, -cam.y)
end

-- Disables the camera, which resets all transforms.
function M.disable(cam)
    love.graphics.pop()
end

-- Sets the position for the camera to center itself on.
function M.setPosition(cam, x, y)
    cam.x = x
    cam.y = y
end

return M