local g = require("globals")
local tm = require("tilemap")
local pm = require("player")
local sm = require("sign")
local cm = require("camera")
local ui = require("ui")

math.randomseed(os.time())

local signTexts = {
    string.upper("Quite a fine day, innilt? This sun is making my relf melt!"),
    string.upper("Press the pause button to pause.")
}

local function loadGraphics()
    love.graphics.setBackgroundColor(1, 1, 1, 1)
    g.IMAGE_SHEET = love.graphics.newImage("assets/sheet.png")
    g.FONT = love.graphics.newFont("assets/font/font.fnt", "assets/font/font.png")
    g.FONT:setLineHeight(1.5)
    love.graphics.setFont(g.FONT)
    ui.load()
end

local function loadAudio()
    g.SFX_VOICE = love.audio.newSource("assets/audio/voice.ogg", "static")
    g.SFX_SELECT = love.audio.newSource("assets/audio/select.ogg", "static")
    g.SFX_CLICK = love.audio.newSource("assets/audio/click.ogg", "static")

    g.MUSIC_LOOP = love.audio.newSource("assets/audio/music_loop.ogg", "stream")
    g.MUSIC_LOOP:setVolume(0.3)
    g.MUSIC_LOOP:setLooping(true)
end

local function spawnEntities(layer)
    for _, tile in ipairs(layer.tiles) do
        local ts = g.tilemap.tileSize

        if tile.id == "9" then
            g.player = pm.newPlayer(tile.x * ts, tile.y * ts)
        elseif tile.id == "10" then
            local poppedText = table.remove(signTexts, 1)
            local sign = sm.newSign(tile.x * ts, tile.y * ts, poppedText)
            table.insert(g.signs, sign)
        end
    end
end

function love.load()
    loadGraphics()
    loadAudio()

    g.tilemap = tm.newTilemapFromFile(
        "assets/tiles/map.json",
        "assets/tiles/spritesheet.png"
    )

    for i, layer in ipairs(g.tilemap.layers) do
        if layer.name == "entities" then
            spawnEntities(layer)
        end
    end

    ui.createPauseMenu()

    g.cam = cm.newCamera()

    g.MUSIC_LOOP:play()
end

function love.update()
    pm.update(g.player)
    
    for _, sign in ipairs(g.signs) do
        sm.update(sign)
    end

    cm.setPosition(
        g.cam,
        g.player.x + g.player.graphic.frameWidth / 2,
        g.player.y + g.player.graphic.frameHeight / 2
    )

    ui.update()
end

function love.draw()
    cm.enable(g.cam)

    for i = #g.tilemap.layers, 1, -1 do
        local layer = g.tilemap.layers[i]
        if layer.name == "walls" or layer.name == "grass" then
            tm.drawLayer(g.tilemap, layer)
        end
    end

    pm.draw(g.player)

    for _, sign in ipairs(g.signs) do 
        sm.draw(sign)
    end

    cm.disable(g.cam)

    ui.draw()
end

function love.keypressed(key, scancode, isrepeat)
    -- return if any of these are true to stop the same input from being used later in the chain
    if pm.handleKeyPressed(g.player, key, scancode, isrepeat) then return end
    if ui.handleKeyPressed(key, scancode, isrepeat) then return end
end

--local str = string.upper("Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua.")