local g = require("globals")
local am = require("animation")
local ui = require("ui")

local M = {}

local function onTextFinished(sign)
    am.play(sign.graphic, "active")
    sign.talking = false
    sign.yscale = 1
    sign.rotation = 0
end

local function playVoice()
    g.SFX_VOICE:stop()

    -- formula here is math.random() * pitch range + lowest pitch possible
    -- so 0.4 and 0.9 give me a range from 0.9 to 1.3
    local pitch = math.random() * 0.4 + 0.9
    g.SFX_VOICE:setPitch(pitch)

    g.SFX_VOICE:play()
end

function M.newSign(x, y, dialogueText)
    local s = {}

    s.graphic = am.newAnimation(g.IMAGE_SHEET, g.TILE_SIZE, g.TILE_SIZE)
    am.add(s.graphic, "idle", {25}, 2)
    am.add(s.graphic, "active", {26, 27, 26, 32}, 2.5)
    am.add(s.graphic, "talk", {33, 26}, 6)
    am.play(s.graphic, "idle")

    s.x = x
    s.y = y

    -- only handles one line of dialogue right now before closing the popup
    s.dialogueText = dialogueText
    s.talking = false
    s.waveTime = 0
    s.voiceTime = 0

    s.xscale = 1
    s.yscale = 1
    s.rotation = 0

    s.onTextFinished = onTextFinished

    return s
end

function M.onInteract(sign, player)
    if not ui.isDialogueVisible() then
        ui.showDialogue(sign.dialogueText, sign)
        am.play(sign.graphic, "talk", true)
        sign.talking = true
        sign.waveTime = love.timer.getTime()
        sign.voiceTime = love.timer.getTime()

        playVoice()
        
        if player.facing == "right" then
            sign.xscale = -1
        elseif player.facing == "left" then
            sign.xscale = 1
        end
    end
end

function M.update(sign)
    if sign.talking then
        local time = love.timer.getTime() - sign.waveTime
        sign.yscale = (math.abs(math.sin(time * 10)) * 0.4) + 0.8
        sign.rotation = math.sin(time * 10) * 0.2

        if love.timer.getTime() - sign.voiceTime > math.random() * 0.2 + 0.1 then
            sign.voiceTime = love.timer.getTime()
            playVoice()
        end
    end

    am.update(sign.graphic)
end

function M.draw(sign)
    local originOffsetX = sign.graphic.frameWidth / 2
    local originOffsetY = sign.graphic.frameHeight

    love.graphics.draw(
        sign.graphic.image,
        sign.graphic.currentQuad,
        sign.x + originOffsetX,
        sign.y + originOffsetY,
        sign.rotation,
        sign.xscale,
        sign.yscale,
        originOffsetX,
        originOffsetY
    )
end

return M