local g = require("globals")
local tm = require("tilemap")
local dm = require("dialogue")

local M = {}

M.ui = {}
M.cursorQuad = nil

function M.load()
    M.cursorQuad = love.graphics.newQuad(128, 0, 16, 16, g.IMAGE_SHEET)
end

function M.createPauseMenu()
    local pm = {}
    pm.panel = tm.getLayerByName(g.tilemap, "pausemenu")
    pm.labels = {"PEKODEX", "PEKOMON", "BAG", "MAKO", "SAVE", "OPTION", "EXIT"}
    pm.cursorPos = 1

    M.ui.pausemenu = pm
    M.ui.dialogue = dm.newDialogue()
end

function M.showDialogue(text, sign)
    dm.show(M.ui.dialogue, text, sign)
end

function M.hideDialogue()
    dm.hide(M.ui.dialogue)
end

function M.advanceDialogue()
    dm.advance(M.ui.dialogue)
end

function M.isDialogueVisible()
    return M.ui.dialogue.visible
end

function M.isDialogueFinished()
    return M.ui.dialogue.textFinished
end

function M.update()
    dm.update(M.ui.dialogue)
end

function M.draw()
    if g.paused then
        local firstTileOnPanel = M.ui.pausemenu.panel.tiles[1]

        local offsetX = love.graphics.getWidth() - 256
        local offsetY = 0
        tm.drawLayer(g.tilemap, M.ui.pausemenu.panel, offsetX, offsetY)

        local panelX = firstTileOnPanel.x * g.TILE_SIZE + offsetX
        local panelY = firstTileOnPanel.y * g.TILE_SIZE + offsetY

        love.graphics.setColor(0, 0, 0, 1)
        for i, label in ipairs(M.ui.pausemenu.labels) do
            love.graphics.print(label, panelX + 50, panelY + 24 + (50 * (i - 1)), 0, 0.5)
        end
        love.graphics.setColor(1, 1, 1, 1)

        love.graphics.draw(g.IMAGE_SHEET, M.cursorQuad, panelX + 20, panelY + 30 + (50 * (M.ui.pausemenu.cursorPos - 1)))
    end

    dm.draw(M.ui.dialogue, M.cursorQuad)
end

function M.handleKeyPressed(key, scancode, isrepeat)
    local handled = false

    if key == "p" and not g.player.moving and not g.dialogueActive then
        g.paused = not g.paused
        M.ui.pausemenu.cursorPos = 1
        g.SFX_SELECT:play()
    end

    if key == "space" and M.isDialogueVisible() then
        if M.isDialogueFinished() then
            M.hideDialogue()
            handled = true
            g.SFX_CLICK:play()
        else
            M.advanceDialogue()
            handled = true
        end
    end

    if g.paused then
        if key == "up" or key == "w" then
            M.ui.pausemenu.cursorPos = M.ui.pausemenu.cursorPos - 1
            if M.ui.pausemenu.cursorPos < 1 then
                M.ui.pausemenu.cursorPos = #M.ui.pausemenu.labels
            end
            g.SFX_CLICK:play()
        elseif key == "down" or key == "s" then
            M.ui.pausemenu.cursorPos = M.ui.pausemenu.cursorPos + 1
            if M.ui.pausemenu.cursorPos > #M.ui.pausemenu.labels then
                M.ui.pausemenu.cursorPos = 1
            end
            g.SFX_CLICK:play()
        end
    end

    return handled
end

return M
